// mainwnd.cpp
//
// Main application window.
//
// $Id: $
//

#include "stdafx.h"
#include "resource.h"
#include "winauth.h"
#include "errorbox.h"

#include "mainwnd.h"
#include "domains.h"
#include "privapi.h"
#include "checkpwd.h"

//---------------------------------------------------------------------------
// OnInitDialog
//
//  Handles WM_INITDIALOG message. Initializes dialog controls.
//
//  Parameters:
//    hWnd   - handle to the window that will receive the focus
//    lParam - dialog creation data
//
//  Returns:
//	  always FALSE.
//
BOOL
CMainDlg::OnInitDialog(
	IN HWND hWnd,
	IN LPARAM lParam
	)
{
	_UNUSED(hWnd);
	_UNUSED(lParam);

	CComboBox wndCombo(GetDlgItem(IDC_DOMAIN));

	// fill domains combo box
	EnumTrustedDomains(EnumDomainProc, (LPARAM)wndCombo.m_hWnd);
	
	if (wndCombo.GetCount() > 0)
		wndCombo.SetCurSel(0);

	// initialize method selection buttons
	CheckRadioButton(IDC_LOGONUSER, IDC_CHANGEPWD, IDC_LOGONUSER);

	// initialize SeTcbPrivilege check box
	CheckDlgButton(IDC_TCB, IsPrivilege(SE_TCB_NAME));
	EnableDlgItem(m_hWnd, IDC_TCB, IsUserAdmin());

	CenterWindow();

	CWindow(GetDlgItem(IDC_USERNAME)).SetFocus();
	return FALSE;
}

//---------------------------------------------------------------------------
// OnTcb
//
//  Handles SeTcbPrivilege check box.
//
//  Parameters:
//	  none.
//
//  Returns:
//	  no return value.
//
void
CMainDlg::OnTcb()
{
	BYTE bBuffer[sizeof(TOKEN_USER) + 8 + 4 * SID_MAX_SUB_AUTHORITIES];
	ULONG cbNeeded;
	HANDLE hToken;
	
	if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken))
		return;

	if (!GetTokenInformation(hToken, TokenUser, bBuffer, sizeof(bBuffer),
						     &cbNeeded))
	{
		CloseHandle(hToken);
		return;
	}

	CloseHandle(hToken);

	PTOKEN_USER pUser = (PTOKEN_USER)bBuffer;

	UINT uCheck = !IsDlgButtonChecked(IDC_TCB);

	if (uCheck)
		_VERIFY(GrantPrivilege(NULL, pUser->User.Sid, NULL, SE_TCB_NAME));
	else
		_VERIFY(RevokePrivilege(NULL, pUser->User.Sid, NULL, SE_TCB_NAME));

	CheckDlgButton(IDC_TCB, uCheck);

	TCHAR szTitle[80];
	TCHAR szMessage[256];

	GetWindowText(szTitle, countof(szTitle));
	AtlLoadString(IDP_RELOGON_REQUIRED, szMessage, countof(szMessage));

	MessageBox(szMessage, szTitle, MB_OK|MB_ICONINFORMATION);
}

//---------------------------------------------------------------------------
// OnLogon
//
//  Handles Logon button.
//
//  Parameters:
//	  none.
//
//  Returns:
//	  no return value.
//
void
CMainDlg::OnLogon()
{
	BOOL bValid = FALSE;
	TCHAR szDomainName[256];
	TCHAR szUserName[256];
	TCHAR szPassword[256];

	GetDlgItemText(IDC_DOMAIN, szDomainName, countof(szDomainName));
	GetDlgItemText(IDC_USERNAME, szUserName, countof(szUserName));
	GetDlgItemText(IDC_PASSWORD, szPassword, countof(szPassword));

	PTSTR pszDomainName = NULL;
	if (szDomainName[0] != 0)
		pszDomainName = szDomainName;

	TCHAR szTitle[80];
	GetWindowText(szTitle, countof(szTitle));

	if (IsDlgButtonChecked(IDC_LOGONUSER))
	{
		bValid = CheckPassword_LogonUser(pszDomainName, szUserName,
										 szPassword, NULL);
	}
	else if (IsDlgButtonChecked(IDC_SSPI))
	{
		bValid = CheckPassword_SSPI(pszDomainName, szUserName, szPassword,
									NULL);
	}
	else
	{
		bValid = CheckPassword_ChangePwd(pszDomainName, szUserName, 
									szPassword, NULL);
	}

	if (!bValid)
	{
		ErrorBox(m_hWnd, _Module.GetResourceInstance(),
				 MAKEINTRESOURCE(IDP_LOGON_FAILED),
				 szTitle, GetLastError());
	}
	else
	{
		TCHAR szMessage[256];
		AtlLoadString(IDP_LOGON_SUCCESSFUL, szMessage, countof(szMessage));

		MessageBox(szMessage, szTitle, MB_OK|MB_ICONINFORMATION);
	}
}

//---------------------------------------------------------------------------
// EnumDomainProc
//
//  This function is called for each trusted domain during domain enumera-
//  tion.
//
//  Parameters:
//	  pszDomainName - domain name
//	  lParam		- contains a handle to the domains combo box
//
//  Returns:
//	  TRUE - to continue enumeration, FALSE - to stop enumeration.
//
BOOL
CALLBACK
CMainDlg::EnumDomainProc(
	IN PCTSTR pszDomainName,
	IN LPARAM lParam
	)
{
	_ASSERTE(pszDomainName != NULL);
	_ASSERTE(lParam != 0);

	CComboBox((HWND)lParam).AddString(pszDomainName);
	return TRUE;
}


