// sspiauth.h
//
// SSPI authentication.
//
// $Id: $
//

#ifndef __sspiauth_h_included
#define __sspiauth_h_included

#define SECURITY_WIN32
#include <security.h>

#ifndef NT_SUCCESS
#	define NT_SUCCESS(x)		(((NTSTATUS)x) >= 0)
#endif

class CSspiBase
{
  public:

	VOID FreeBuffer(
		IN PVOID pBuffer
		)
	{ LocalFree((HLOCAL)pBuffer); }

  protected:

	HINSTANCE				m_hSecDll;
	PSecurityFunctionTable	m_pEntryTable;
	ULONG					m_cbMaxToken;
    SecHandle				m_hCtx;
    CredHandle				m_hCred;
    bool					m_bHaveCredHandle;
    bool					m_bHaveCtxHandle;

  protected:

	CSspiBase();
	~CSspiBase();

	ULONG Initialize(
		IN PCTSTR pszProvider
		);

	ULONG CompleteToken(
		IN SECURITY_STATUS Status,
		IN PSecBufferDesc pBufDesc,
		OUT PVOID * ppToken,
		OUT PULONG pcbToken
		);

};

class CSspiClient : public CSspiBase
{
  public:

	ULONG Initialize(
		IN PCTSTR pszDomainName,
		IN PCTSTR pszUserName,
		IN PCTSTR pszPassword,
		IN PCTSTR pszProvider = _T("NTLM")
		);

	ULONG Start(
		OUT PVOID * ppRequest,
		OUT PULONG pcbRequest
		);

	ULONG Continue(
		IN PVOID pResponse,
		IN ULONG cbResponse,
		OUT PVOID * ppRequest,
		OUT PULONG pcbRequest
		);

};

class CSspiServer : public CSspiBase
{
  public:

	ULONG Initialize(
		IN PCTSTR pszProvider = _T("NTLM")
		);

	ULONG Continue(
		IN PVOID pRequest,
		IN ULONG cbRequest,
		OUT PVOID * ppResponse,
		OUT PULONG pcbResponse
		);

	ULONG GetToken(
		OUT PHANDLE hToken,
		IN ULONG dwAccess
		);

	ULONG CheckGuest();
	  
};

#endif // __sspiauth_h_included
