// winauth.cpp
//
// Entry point and utility functions.
//
// $Id: $
//

#include "stdafx.h"
#include "resource.h"
#include "winauth.h"
#include "mainwnd.h"

#pragma comment(lib, "netapi32.lib")

CAppModule		_Module;
OSVERSIONINFO	_osvi;

//---------------------------------------------------------------------------
// WinMain
//
//  Program entry point.
//
//  Parameters:
//    hInstance     - instance handle
//    hPrevInstance - previous instance handle
//    pszCmdLine    - command line
//    nShowCmd	    - show command
//
//  Returns:
//    the return value is the exit code of the process.
//
EXTERN_C
int
WINAPI
_tWinMain(
    IN HINSTANCE hInstance,
    IN HINSTANCE hPrevInstance,
    IN PTSTR pszCmdLine,
    IN int nShowCmd
    )
{
    _UNUSED(hPrevInstance);
	_UNUSED(pszCmdLine);
	_UNUSED(nShowCmd);

	_osvi.dwOSVersionInfoSize = sizeof(_osvi);

	// determine operating system version
	_VERIFY(GetVersionEx(&_osvi));

    _Module.Init(NULL, hInstance);
    InitCommonControls();

	{
		CMainDlg dlgMain;
		dlgMain.DoModal();
	}

    _Module.Term();

	_CrtDumpMemoryLeaks();
    return 0;
}

//---------------------------------------------------------------------------
// EnableDlgItem
//
//  Enables or disables a dialog control.
//
//  Parameters:
//	  hWnd	  - dialog window identifier
//	  nCtrlId - control identifier
//	  bEnable - enable flag
//
//  Returns:
//	  TRUE, if successful, FALSE - otherwise.
//
EXTERN_C
BOOL
APIENTRY
EnableDlgItem(
	HWND hWnd,
	UINT nCtrlId,
	BOOL bEnable
	)
{
	_ASSERTE(IsWindow(hWnd));
	_ASSERTE(nCtrlId != 0);

	hWnd = GetDlgItem(hWnd, nCtrlId);
	if (hWnd != NULL)
	{
		EnableWindow(hWnd, bEnable);
		return TRUE;
	}

	return FALSE;
}

//---------------------------------------------------------------------------
// ShowDlgItem
//
//  Shows or hides the specified dialog control.
//
//  Parameters:
//	  hWnd     - dialog window handle
//	  nCtrlId  - control identifier
//	  nCmdShow - show command
//
//  Returns:
//	  TRUE, if successful, FALSE - otherwise.
//
EXTERN_C
BOOL
APIENTRY
ShowDlgItem(
	IN HWND hWnd,
	IN UINT nCtrlId,
	IN INT nCmdShow
	)
{
	_ASSERTE(IsWindow(hWnd));
	_ASSERTE(nCtrlId != 0);

	hWnd = GetDlgItem(hWnd, nCtrlId);
	if (hWnd != NULL)
	{
		ShowWindow(hWnd, nCmdShow);
		return TRUE;
	}

	return FALSE;
}

//---------------------------------------------------------------------------
// FormatDlgItem
//
//  Formats the dialog item text in according to the format string and
//  additional arguments.
//
//  Parameters:
//    hWnd      - dialog window identifier
//    nCtrlID   - control identifier
//    pszFormat - pointer to the format string or the identifier of the
//		  string in module resources
//    ...       - additional argiments
//
//  Returns:
//    no return value.
//
EXTERN_C
VOID 
CDECL
FormatDlgItem(
    HWND hWnd, 
    UINT nCtrlID, 
    PCTSTR pszFormat, 
    ...
    )
{
    _ASSERTE(IsWindow(hWnd));
    _ASSERTE(nCtrlID != 0);

    // load the format string if it is not specified
    TCHAR szFormat[256];
    if (pszFormat == NULL)
    {
		GetDlgItemText(hWnd, nCtrlID, szFormat, countof(szFormat));
		pszFormat = szFormat;
    }
    else if (HIWORD(pszFormat) == 0)
    {
		LoadString(_Module.GetResourceInstance(), LOWORD(pszFormat), 
				   szFormat, countof(szFormat));
		pszFormat = szFormat;
    }

    _ASSERTE(pszFormat != NULL);
    _ASSERTE(HIWORD(pszFormat) != 0);

    TCHAR szNewText[256];

    // format the new text of the control
    va_list va;
    va_start(va, pszFormat);

    FormatMessage(FORMAT_MESSAGE_FROM_STRING, pszFormat, 0, 0, 
		  szNewText, countof(szNewText), &va);

    va_end(va);

    // set the new text
    SetDlgItemText(hWnd, nCtrlID, szNewText);
}

//---------------------------------------------------------------------------
// FormatString
//
//  Formats the string with FormatMessage rules, using an explicitly speci-
//  fied string or a string in module resources as a source.
//
//  Parameters:
//    pszBuffer - pointer to a buffer that receives the formatted string
//    cchMax    - size of the buffer in bytes
//    pszFormat - pointer to the format string or the identifier of the
//		          string in module resources
//
//  Returns:
//    number of characters written into the output buffer not including the
//    terminating null character.
//
EXTERN_C
int 
__cdecl 
FormatString(
    LPTSTR pszBuffer, 
    UINT cchMax, 
    LPCTSTR pszFormat, 
    ...
    )
{
    // if a string identifier was specified, load the string from the
    // resources first
    TCHAR szFormat[1024];
    if (HIWORD(pszFormat) == 0)
    {
		AtlLoadString(LOWORD(pszFormat), szFormat, countof(szFormat));
		pszFormat = szFormat;
    }

    // format the string
    va_list va;
    va_start(va, pszFormat);

    int cch = FormatMessage(FORMAT_MESSAGE_FROM_STRING, pszFormat, 0, 0,
			    pszBuffer, cchMax, &va);

    va_end(va);

    return cch;
}

